<?php


/**
 * @file
 * Contains \Drupal\dsfr_twig_components\Twig\RenderBlock.
 */


namespace Drupal\dsfr_twig_components\Twig;

use Drupal\file\Entity\File;
use Drupal\media\Entity\Media;
use Symfony\Component\Mime\MimeTypes;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;


/**
 * Adds extension to render a block.
 */
class DsfrTwigComponents extends AbstractExtension {


  private $utf8 = 'UTF-8';


  /**
   * {@inheritdoc}
   */
  public function getName() {


    return 'dsfr_twig_components';
  }


  /**
   * {@inheritdoc}
   */
  public function getFunctions() {


    return [

      new TwigFunction(
        'dsfr_badge',
        [$this, 'dsfr_badge'],
        ['is_safe' => ['html']]
      ),
      new TwigFunction(
        'dsfr_button',
        [$this, 'dsfr_button'],
        ['is_safe' => ['html']]
      ),
      new TwigFunction(
        'dsfr_callout',
        [$this, 'dsfr_callout'],
        ['is_safe' => ['html']]
      ),
      new TwigFunction(
        'dsfr_download',
        [$this, 'dsfr_download'],
        ['is_safe' => ['html']]
      ),
      new TwigFunction(
        'dsfr_highlight',
        [$this, 'dsfr_highlight'],
        ['is_safe' => ['html']]
      ),
      new TwigFunction(
        'dsfr_link',
        [$this, 'dsfr_link'],
        ['is_safe' => ['html']]
      ),
      new TwigFunction(
        'dsfr_table',
        [$this, 'dsfr_table'],
        ['is_safe' => ['html']]
      ),
      new TwigFunction(
        'get_media_info',
        [$this, 'get_media_info'],
        ['is_safe' => ['html']]
      ),
      new TwigFunction(
        'limit_string',
        [$this, 'limit_string'],
        ['is_safe' => ['html']]
      ),
    ];
  }

  /**
   *
   */
  public function dsfr_badge( $text, $class = NULL ) {

    /*
      See: https://www.systeme-de-design.gouv.fr/elements-d-interface/composants/badge
      Class examples:
        - fr-badge--info, fr-badge--success, fr-badge--warning, fr-badge--error, fr-badge--new
        - fr-badge--no-icon
        - fr-badge--green-menthe
        - fr-badge--sm
    */
    if( $class != NULL ) { $class = ' ' . $class; }

    return '<p class="fr-badge'.$class.'">'.$text.'</p>';
  }


  /**
   *
   */
  public function dsfr_button(
    $label = NULL,
    $class = NULL,
    $title = NULL,
    $is_disabled = false )
  {
   
    /*
      See: https://www.systeme-de-design.gouv.fr/elements-d-interface/composants/bouton
      Class examples:
        - fr-btn--secondary, fr-btn--tertiary
        - fr-btn--icon-left fr-icon-checkbox-circle-line
    */
    if( $class != NULL ) { $class = ' ' . $class; }
    if( $title != NULL ) { $title = ' title="' . $title .'"'; }
    $disabled = ( $is_disabled == true ) ? 'disabled' : '';


    if( $label != NULL ) {


      return '
      <button class="fr-btn'.$class.'"'.$title.$disabled.'>
      '. $label .'
      </button>
      ';
    }
  }


  /**
   * DSFR callout
   */
  public function dsfr_callout(
    $text,
    $title = NULL,
    $title_type = 'h3',
    $icon = 'fr-icon-information-line',
    $class = NULL,
    $button_args = [
      'label' => NULL,
      'class' => NULL,
      'title' => NULL,
      'is_disabled' => false
    ])
  {


    $callout_title = ( $title != NULL ) ? '<'.$title_type.' class="fr-callout__title">'. $title .'</'.$title_type.'>' : '';
    $callout_button = $this->dsfr_button(
      $button_args['label'],
      $button_args['class'],
      $button_args['title'],
      $button_args['is_disabled']
    );


    /*
      See: https://www.systeme-de-design.gouv.fr/elements-d-interface/composants/mise-en-avant
      Class examples:
        - fr-btn--secondary, fr-btn--tertiary
        - fr-btn--icon-left fr-icon-checkbox-circle-line
    */
    if( $class != NULL ) { $class = ' '.$class; }


    return '
    <div class="fr-callout '.$icon.$class.'">    
      '. $callout_title .'
      <p class="fr-callout__text">'. $text .'</p>  
      '. $callout_button .'
    </div>
    ';
  }
 
  /**
   * DSFR download file (link)
   * See: https://www.systeme-de-design.gouv.fr/elements-d-interface/composants/lien
   */
  public function dsfr_download (
    $mid,
    $wrapper = 0,
    $args = [] )
  {


    if ( is_numeric( $mid ) ) {


      $file = $this->get_media_info( $mid );
 
      // Card
      if( $wrapper == 1 ) {
 
        return '      
        <div class="fr-card fr-enlarge-link fr-card--download">
          <div class="fr-card__body">
            <div class="fr-card__content">
              <h3 class="fr-card__title">
                <a download href="'. $file['url'] .'">
                '. t('Download the document') .'  '. $file['name'] .'
                </a>
              </h3>
              <p class="fr-card__desc"></p>
              <div class="fr-card__end">
                <p class="fr-card__detail">'. $file['ext'] .' – '. $file['size'] .'</p>
              </div>
            </div>
          </div>
        </div>';
 
      // Tile (with picto)
      } elseif ( $wrapper == 2 ) {


        $lib = base_path() . 'themes/contrib/dsfr/dist/dsfr/';
        // to do, add: '. t('Detail (optional)') .'
 
        return '
        <div class="fr-tile fr-tile--download fr-enlarge-link" id="tile-'.time().'">
          <div class="fr-tile__body">
            <div class="fr-tile__content">
              <h3 class="fr-tile__title">
                <a href="'. $file['url'] .'" download>'. t('Download the document') .' '. $file['name'] .'</a>
              </h3>
              <p class="fr-tile__detail"></p>
            </div>
          </div>
          <div class="fr-tile__header">
            <div class="fr-tile__pictogram">
              <svg aria-hidden="true" class="fr-artwork" viewbox="0 0 80 80" width="80px" height="80px">
                <use class="fr-artwork-decorative" href="'. $lib .'artwork/pictograms/buildings/city-hall.svg#artwork-decorative"></use>
                <use class="fr-artwork-minor" href="'. $lib .'artwork/pictograms/buildings/city-hall.svg#artwork-minor"></use>
                <use class="fr-artwork-major" href="'. $lib .'artwork/pictograms/buildings/city-hall.svg#artwork-major"></use>
              </svg>
            </div>
          </div>
        </div>';
 
      // Link
      } else {
 
        return '    
        <a class="fr-link fr-link--download" download href="'. $file['url']  .'">
          '. $file['name'] .'
          <span class="fr-link__detail">'. $file['ext'] .' – '. $file['size'].'</span>
        </a>
        ';
      }
   
    } else {


      return t('The MID should be an number');
    }
  }


  /**
   *
   */
  public function dsfr_highlight( $text, $class = NULL ) {


    /*
      See: https://www.systeme-de-design.gouv.fr/elements-d-interface/composants/mise-en-exergue
      Class examples:
        - fr-highlight--green-emeraude
    */
    if( $class != NULL ) { $class = ' ' . $class; }


    return '
    <div class="fr-highlight'.$class.'">
      <p>'.$text.'</p>
    </div>
    ';
  }


  /**
   * DSFR Link
   * Link for download, see: dsfr_download()
   */
  public function dsfr_link(
    $label,
    $link = NULL,
    $class = NULL,
    $external = false)
  {
   
    $url = ( $link != NULL ) ? $link  : '#';
    $target = ( $external == true ) ? '_blank' : '_self';


    /*
      See: https://www.systeme-de-design.gouv.fr/elements-d-interface/composants/lien
      Class examples:
        - fr-icon-arrow-right-line fr-link--icon-right
        - fr-link--sm, fr-link--lg
    */
    if( $class != NULL ) { $class = ' class="' . $class .'" '; }
   
    return '<a href="'.$url.'"'.$class.'target="'.$target.'">'.$label.'</a>';
  }


  /**
   *
   */
  public function dsfr_table( $table, $caption = NULL, $class = NULL ) {


    /* Table pattern :


    $table = [


      [ 'title' => 'Titre',
        'content' => [ 'Data1', 'Data2']
      ],
      [ 'title' => 'Titre',
        'content' => [ 'Data3', 'Data4']
      ],
      [ 'title' => 'Titre',
        'content' => [ 'Data5', 'Data6']
      ],
      [ 'title' => 'Titre',
        'content' => [ 'Data7', 'Data8']
      ],
      [ 'title' => 'Titre',
        'content' => [ 'Data9', 'Data10']
      ],
    ];


    or with twig:
   
    {% set table = [


  {
    'title': 'Titre',
    'content': ['Data1', 'Data2'],
  },
  {
    'title': 'Titre',
    'content': ['Data3', 'Data4'],
  },
  {
    'title': 'Titre',
    'content': ['Data5', 'Data6'],
  },
  {
    'title': 'Titre',
    'content': ['Data7', 'Data8'],
  },
  {
    'title': 'Titre',
    'content': ['Data9', 'Data10'],
  }
] %}


    */


    if( count( $table ) > 0  ) {


      $tbody_memory = [];
      $thead = '<thead><tr>';
      $tbody= '<tbody>';
 
      foreach ($table as $row ) {
 
        $title = $row['title'];
        $thead .= '<th scope="col">'. $title .'</th>';
 
        for( $i = 0; $i < count($row['content']); $i ++) {
 
          $col = 'col_' . $i;
 
          $tbody_memory[$col][] = $row['content'][$i];
        }
      }
 
      foreach ( $tbody_memory as $key => $values ) {
 
        $tbody .= '<tr>';
        foreach ( $values as $value ) {
          $tbody .=  '<td>'. $value .'</td>';
        }
        $tbody .= '</tr>';
      }
 
      $thead.= '</tr></thead>';
      $tbody.= '</tbody>';
 
      if( $caption != NULL ) { $caption = '<caption>' . $caption . '</caption>'; }
     
      /*
        See: https://www.systeme-de-design.gouv.fr/elements-d-interface/composants/tableau/
        Class examples:
          - fr-highlight--green-emeraude
      */
      if( $class != NULL ) { $class = ' ' . $class; }
 
      return '
      <div class="fr-table">    
        <table>        
          '. $caption .'
          '. $thead .'        
          '. $tbody .'  
        </table>
      </div>';
    }
  }


  /**
   * Convert a file size to make it more readable
   */
  public function convert_bytes( $octet ) {
  
    $unite = array('octet', 'ko', 'mo', 'go');
  
    if ($octet < 1000) // octet
    {
      return $octet . $unite[0];
    } else {
      if ($octet < 1000000) // ko
      {
        $ko = round($octet / 1024, 2);
        return $ko . $unite[1];
      } else // Mo ou Go
      {
        if ($octet < 1000000000) // Mo
        {
          $mo = round($octet / (1024 * 1024), 2);
          return $mo . $unite[2];
        } else // Go
        {
          $go = round($octet / (1024 * 1024 * 1024), 2);
          return $go . $unite[3];
        }
      }
    }
  }
 
  /**
   * Get file information from a media ID
   */
  public function get_media_info( $mid ) {


    $media = Media::load($mid);
    $fid = $media->getSource()->getSourceFieldValue($media);
    $file = File::load($fid);
    $mime = $file->getMimeType();
   
    $type = New MimeTypes();
    $ext = $type->getExtensions( $mime );
   
    return [
      'name' => $media->getName(),
      'url' => $file->createFileUrl(),
      'size' => $this->convert_bytes( $file->getSize() ),
      'mime' => $mime,
      'ext' => strtoupper( $ext[0] ),
    ];
  }


  /**
   * Limiting a number of characters
   */
  public function limit_string($str = '', $limit = 60) {

    return (strlen($str) > $limit) ? mb_substr($str, 0, $limit, $this->utf8) . '[...]' : $str;
  }
}